package com.jiuqi.util;

import java.util.ArrayList;

/* loaded from: com_jiuqi_util.jar:com/jiuqi/util/SequencialList.class */
public class SequencialList {
    private static final int NODE_SIZE = 1024;
    private static final int NODE_SHIFT = 10;
    private static final int NODE_OFFSET = 1023;
    private int size;
    private Object[] firstNode;
    private Object[] lastNode;
    private Object[] cacheNode;
    private int cachePage;

    public SequencialList() {
        clear();
    }

    public void clear() {
        this.size = 0;
        while (this.firstNode != null) {
            this.lastNode = (Object[]) this.firstNode[NODE_SIZE];
            this.firstNode[NODE_SIZE] = null;
            this.firstNode = this.lastNode;
        }
        this.lastNode = null;
        this.cacheNode = null;
    }

    public void add(Object value) {
        int offset = this.size & NODE_OFFSET;
        if (offset == 0) {
            Object[] newNode = new Object[1025];
            if (this.lastNode != null) {
                this.lastNode[NODE_SIZE] = newNode;
                this.lastNode = newNode;
            } else {
                this.firstNode = newNode;
                this.lastNode = newNode;
            }
        }
        this.lastNode[offset] = value;
        this.size++;
    }

    public int size() {
        return this.size;
    }

    public Object get(int index) {
        if (index < 0 || index >= this.size) {
            return null;
        }
        if (this.cacheNode != null) {
            int page = index >> NODE_SHIFT;
            if (page == this.cachePage) {
                return this.cacheNode[index & NODE_OFFSET];
            }
            if (page > this.cachePage) {
                index -= this.cachePage << NODE_SHIFT;
            } else {
                this.cachePage = 0;
                this.cacheNode = this.firstNode;
            }
        } else {
            this.cachePage = 0;
            this.cacheNode = this.firstNode;
        }
        while (index >= NODE_SIZE) {
            index -= 1024;
            this.cacheNode = (Object[]) this.cacheNode[NODE_SIZE];
            this.cachePage++;
        }
        return this.cacheNode[index];
    }

    public void set(int index, Object value) {
        if (index < 0 || index >= this.size) {
            return;
        }
        if (this.cacheNode != null) {
            int page = index >> NODE_SHIFT;
            if (page == this.cachePage) {
                this.cacheNode[index & NODE_OFFSET] = value;
                return;
            } else if (page > this.cachePage) {
                index -= this.cachePage << NODE_SHIFT;
            } else {
                this.cachePage = 0;
                this.cacheNode = this.firstNode;
            }
        } else {
            this.cachePage = 0;
            this.cacheNode = this.firstNode;
        }
        while (index >= NODE_SIZE) {
            index -= 1024;
            this.cacheNode = (Object[]) this.cacheNode[NODE_SIZE];
            this.cachePage++;
        }
        this.cacheNode[index] = value;
    }

    public int indexOf(Object data) {
        int result = 0;
        if (data != null) {
            Object[] objArr = this.firstNode;
            while (true) {
                Object[] node = objArr;
                if (result < this.size) {
                    int count = this.size - result >= NODE_SIZE ? NODE_SIZE : this.size - result;
                    for (int i = 0; i < count; i++) {
                        if (data.equals(node[i])) {
                            return result + i;
                        }
                    }
                    result += NODE_SIZE;
                    objArr = (Object[]) node[NODE_SIZE];
                } else {
                    return -1;
                }
            }
        } else {
            Object[] objArr2 = this.firstNode;
            while (true) {
                Object[] node2 = objArr2;
                if (result < this.size) {
                    int count2 = this.size - result >= NODE_SIZE ? NODE_SIZE : this.size - result;
                    for (int i2 = 0; i2 < count2; i2++) {
                        if (node2[i2] == null) {
                            return result + i2;
                        }
                    }
                    result += NODE_SIZE;
                    objArr2 = (Object[]) node2[NODE_SIZE];
                } else {
                    return -1;
                }
            }
        }
    }

    public int remove(Object data) {
        int index = indexOf(data);
        if (index >= 0) {
            remove(index);
        }
        return index;
    }

    public Object remove(int index) {
        if (index < 0 || index >= this.size) {
            return null;
        }
        int tailCount = (this.size - index) - 1;
        if (this.cacheNode != null) {
            int page = index >> NODE_SHIFT;
            if (page == this.cachePage) {
                index &= NODE_OFFSET;
            } else if (page > this.cachePage) {
                index -= this.cachePage << NODE_SHIFT;
            } else {
                this.cachePage = 0;
                this.cacheNode = this.firstNode;
            }
        } else {
            this.cachePage = 0;
            this.cacheNode = this.firstNode;
        }
        while (index >= NODE_SIZE) {
            index -= 1024;
            this.cacheNode = (Object[]) this.cacheNode[NODE_SIZE];
            this.cachePage++;
        }
        Object result = this.cacheNode[index];
        Object[] node = this.cacheNode;
        while (tailCount > 0) {
            int count = index + tailCount;
            if (count > NODE_SIZE) {
                count = NODE_SIZE;
            }
            for (int i = index; i < count; i++) {
                node[i] = node[i + 1];
            }
            tailCount = (tailCount - count) + index;
            if (tailCount > 0) {
                index = 0;
                Object[] nextNode = (Object[]) node[NODE_SIZE];
                node[NODE_OFFSET] = nextNode[0];
                node = nextNode;
            }
        }
        this.size--;
        if ((this.size & NODE_OFFSET) == 0) {
            this.lastNode = node;
            node[NODE_SIZE] = null;
        }
        return result;
    }

    public ArrayList toArrayList() {
        ArrayList result = new ArrayList(this.size);
        for (int i = 0; i < this.size; i++) {
            result.add(get(i));
        }
        return result;
    }

    public static void test1() {
        ArrayList list1 = new ArrayList();
        SequencialList list2 = new SequencialList();
        long curTime = System.currentTimeMillis();
        for (int i = 0; i < 1000000; i++) {
            list1.add(new Integer(i));
        }
        System.out.print("ArrayList Create: ");
        System.out.println(System.currentTimeMillis() - curTime);
        long curTime2 = System.currentTimeMillis();
        for (int i2 = 0; i2 < 1000000; i2++) {
            list2.add(new Integer(i2));
        }
        System.out.print("SequencialList Create: ");
        System.out.println(System.currentTimeMillis() - curTime2);
        long curTime3 = System.currentTimeMillis();
        for (int k = 0; k < NODE_SHIFT; k++) {
            for (int i3 = 0; i3 < 1000000; i3++) {
                if (((Integer) list1.get(i3)).intValue() != i3) {
                    System.out.println("Error");
                }
            }
        }
        System.out.print("ArrayList Get: ");
        System.out.println(System.currentTimeMillis() - curTime3);
        long curTime4 = System.currentTimeMillis();
        for (int k2 = 0; k2 < NODE_SHIFT; k2++) {
            for (int i4 = 0; i4 < 1000000; i4++) {
                if (((Integer) list2.get(i4)).intValue() != i4) {
                    System.out.println("Error");
                }
            }
        }
        System.out.print("SequencialList Get: ");
        System.out.println(System.currentTimeMillis() - curTime4);
        long curTime5 = System.currentTimeMillis();
        for (int i5 = 0; i5 < 1000000; i5 += 3000) {
            if (list1.indexOf(list1.get(i5)) != i5) {
                System.out.println("Error");
            }
        }
        System.out.print("ArrayList indexOf: ");
        System.out.println(System.currentTimeMillis() - curTime5);
        long curTime6 = System.currentTimeMillis();
        for (int i6 = 0; i6 < 1000000; i6 += 3000) {
            if (list2.indexOf(list2.get(i6)) != i6) {
                System.out.println("Error");
            }
        }
        System.out.print("SequencialList indexOf: ");
        System.out.println(System.currentTimeMillis() - curTime6);
        long curTime7 = System.currentTimeMillis();
        for (int i7 = 0; i7 < 1000000; i7 += 1000) {
            list1.remove(i7);
        }
        System.out.print("ArrayList remove: ");
        System.out.println(System.currentTimeMillis() - curTime7);
        long curTime8 = System.currentTimeMillis();
        for (int i8 = 0; i8 < 1000000; i8 += 1000) {
            list2.remove(i8);
        }
        System.out.print("SequencialList remove: ");
        System.out.println(System.currentTimeMillis() - curTime8);
    }

    public static void test2() {
        SequencialList list = new SequencialList();
        for (int i = 0; i < 4000; i++) {
            list.add(String.valueOf(i));
        }
        for (int i2 = 0; i2 < 4000; i2++) {
            list.get(i2);
        }
        for (int i3 = 0; i3 < 4000; i3++) {
            list.set(i3 * 2, null);
        }
        for (int i4 = 400; i4 < 4000; i4++) {
            System.out.println(list.get(i4));
        }
    }

    public static void main(String[] args) {
        test1();
    }
}
